// tmtlib@narod.ru

/* Includes ------------------------------------------------------------------*/
#include "stm32f10x.h"
#include "stm32f10x_gpio.h"
#include "stm32f10x_rcc.h"
#include "stm32f10x_adc.h"
#include "stm32f10x_dma.h"
#include "stm32f10x_usart.h"
#include "stm32f10x_tim.h"
#include "misc.h"

/** @addtogroup STM32F10x_StdPeriph_Examples
  * @{
  */

/** @addtogroup ADC_ADC1_DMA
  * @{
  */ 


/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
#define ADC1_DR_Address    ((uint32_t)0x4001244C)

/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/

//uint16_t ADC_Val; //Stores the calculated ADC value
double voltage1; //Used to store the actual voltage calculated by ADC for the 1st channel
double voltage2; //Used to store the actual voltage calculated by ADC for the 2nd channel
__IO uint16_t buffer1[2048];
__IO uint16_t buffer2[2048];

int needcopy=0;
unsigned char zdata;


//1024+1024

ADC_InitTypeDef ADC_InitStructure;
DMA_InitTypeDef DMA_InitStructure;

//__IO uint16_t ADCConvertedValue;

RCC_ClocksTypeDef RCC_Clocks;

TIM_TimeBaseInitTypeDef TIM_TimeBaseStructure;
//TIM_OCInitTypeDef zzz;

NVIC_InitTypeDef NVIC_InitStructure;


/* Private function prototypes -----------------------------------------------*/
void RCC_Configuration(void);
void GPIO_Configuration(void);
void USART1_Configuration(void);
void UARTSend1(const char *pucBuffer);
void print_dex(const int ivalue);


char* itoa(int val, int base) // ڧݧ  ܧ
 {

 static char buf[32] = {0};

 int i = 30;

 for(; val && i ; --i, val /= base)

 buf[i] = "0123456789abcdef"[val % base];

 return &buf[i+1];

 }

/* Private functions ---------------------------------------------------------*/

/**
  * @brief   Main program
  * @param  None
  * @retval None
  */
void adc1_dma(void)
{
  int i, AD_value1, AD_value2;
  

  /*!< At this stage the microcontroller clock setting is already configured, 
       this is done through SystemInit() function which is called from startup
       file (startup_stm32f10x_xx.s) before to branch to application main.
       To reconfigure the default setting of SystemInit() function, refer to
       system_stm32f10x.c file
     */     
       
  /* System clocks configuration ---------------------------------------------*/
  RCC_Configuration();

  /* GPIO configuration ------------------------------------------------------*/
  GPIO_Configuration();

  USART1_Configuration();  

  //??????
  RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM3, ENABLE);


   /***************TIMER init**************************/



  TIM_TimeBaseStructInit(&TIM_TimeBaseStructure);
    TIM_TimeBaseStructure.TIM_RepetitionCounter = 0;
    TIM_TimeBaseStructure.TIM_Period = 1;
    TIM_TimeBaseStructure.TIM_Prescaler =0x00c7; // 0x018f;
    TIM_TimeBaseStructure.TIM_ClockDivision = TIM_CKD_DIV1;
    TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;

    TIM_TimeBaseInit(TIM3, &TIM_TimeBaseStructure);

    TIM_SelectOutputTrigger(TIM3, TIM_TRGOSource_Update);


    /*************IRQ**************/





        /* Configure and enable ADC interrupt */

        NVIC_InitStructure.NVIC_IRQChannel = DMA1_Channel1_IRQn;

        NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;

        NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;

        NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;

        NVIC_Init(&NVIC_InitStructure);


  /* DMA1 channel1 configuration ----------------------------------------------*/
  DMA_DeInit(DMA1_Channel1);
  DMA_InitStructure.DMA_PeripheralBaseAddr = ADC1_DR_Address;
  DMA_InitStructure.DMA_MemoryBaseAddr = (uint32_t)& buffer1;
  DMA_InitStructure.DMA_DIR = DMA_DIR_PeripheralSRC;
  DMA_InitStructure.DMA_BufferSize = 2048; //!! 1024+1024
  DMA_InitStructure.DMA_PeripheralInc = DMA_PeripheralInc_Disable;
  DMA_InitStructure.DMA_MemoryInc = DMA_MemoryInc_Enable; //?
  DMA_InitStructure.DMA_PeripheralDataSize = DMA_PeripheralDataSize_HalfWord;
  DMA_InitStructure.DMA_MemoryDataSize = DMA_MemoryDataSize_HalfWord;
  DMA_InitStructure.DMA_Mode = DMA_Mode_Circular;
  DMA_InitStructure.DMA_Priority = DMA_Priority_High;
  DMA_InitStructure.DMA_M2M = DMA_M2M_Disable;

  //DMA_InitStructure.DMA_FIFOThreshold = DMA_FIFOThreshold_HalfFull;

  DMA_Init(DMA1_Channel1, &DMA_InitStructure);
  
  DMA_ITConfig(DMA1_Channel1, DMA_IT_TC|DMA_IT_HT, ENABLE);


  /* Enable DMA1 channel1 */
  DMA_Cmd(DMA1_Channel1, ENABLE);
  
  /* ADC1 configuration ------------------------------------------------------*/
  ADC_InitStructure.ADC_Mode = ADC_Mode_Independent;
  ADC_InitStructure.ADC_ScanConvMode = ENABLE;
  ADC_InitStructure.ADC_ContinuousConvMode = DISABLE; //????ENABLE;
  ADC_InitStructure.ADC_ExternalTrigConv =  ADC_ExternalTrigConv_T3_TRGO; //???ADC_ExternalTrigConv_None;
  ADC_InitStructure.ADC_DataAlign = ADC_DataAlign_Right;
  ADC_InitStructure.ADC_NbrOfChannel = 2; //!!
  ADC_Init(ADC1, &ADC_InitStructure);

  /* ADC1 regular channel14 configuration */ 
  ADC_RegularChannelConfig(ADC1, ADC_Channel_14, 1, ADC_SampleTime_55Cycles5);
  ADC_RegularChannelConfig(ADC1, ADC_Channel_4, 2, ADC_SampleTime_55Cycles5);

  /* Enable ADC1 DMA */
  ADC_DMACmd(ADC1, ENABLE);
  
  /* Enable ADC1 */
  ADC_Cmd(ADC1, ENABLE);

  /* Enable ADC1 reset calibration register */   
  ADC_ResetCalibration(ADC1);
  /* Check the end of ADC1 reset calibration register */
  while(ADC_GetResetCalibrationStatus(ADC1));

  /* Start ADC1 calibration */
  ADC_StartCalibration(ADC1);
  /* Check the end of ADC1 calibration */
  while(ADC_GetCalibrationStatus(ADC1));
     
  /* Start ADC1 Software Conversion */ 
  //???ADC_SoftwareStartConvCmd(ADC1, ENABLE);

  ADC_ExternalTrigConvCmd(ADC1, ENABLE);
  TIM_Cmd(TIM3, ENABLE);
  
  UARTSend1(" The ADC conversion value is : \r\n");

  int ii;
  while (1)
  {
	  if ((USART1->SR & USART_FLAG_RXNE) != (u16)RESET)
	         {
               ii = USART_ReceiveData(USART1);
               //UARTSend((const unsigned char*)&ii,1);
               if (ii==0x01) {needcopy=1;

               ii=0x1234;
               //UARTSend1((const unsigned char*)&ii);
               UARTSend((const unsigned char*)&ii,2);
               }
	         }


/*
    UARTSend1(" The ADC conversion value 1 is : \r\n");
    print_dex(AD_value1);
    UARTSend1(" The ADC conversion value 2 is : \r\n");
    print_dex(AD_value2);
*/

    /*
    RCC_GetClocksFreq (&RCC_Clocks);
    UARTSend1(" SYSAAA ");
    UARTSend1(itoa(RCC_Clocks.SYSCLK_Frequency,10));
    UARTSend1(" HCLKAAA ");
    UARTSend1(itoa(RCC_Clocks.HCLK_Frequency,10));
    */

  }
}

void UARTSend(const unsigned char *pucBuffer, unsigned long ulCount)
{
    //
    // Loop while there are more characters to send.
    //
    while(ulCount--)
    {
        USART_SendData(USART1, (uint16_t) *pucBuffer++);
        /* Loop until the end of transmission */
        while(USART_GetFlagStatus(USART1, USART_FLAG_TC) == RESET)
        {
        }
    }
}

DMA1_Channel1_IRQHandler(void)
{
	int i, AD_value1, AD_value2;
	AD_value1 = buffer1[0];
	AD_value2 = buffer1[1];


	if (DMA_GetITStatus(DMA1_IT_HT1))
	 {
	 DMA_ClearITPendingBit(DMA1_IT_HT1);

	 if (needcopy==1)
 	 	 {
 		 	 needcopy=0;

 		 	 for(i=0; i<1024; i++) buffer2[i]=buffer1[i];
 		     for(i=0; i<1024; i++) UARTSend((const unsigned char*)&buffer2[i],2);

 	 	 }

	// UARTSend1(" The ADC conversion value 1 is : \r\n");
	//     print_dex(AD_value1);
	 }

	 if (DMA_GetITStatus(DMA1_IT_TC1))
	 {
	 DMA_ClearITPendingBit(DMA1_IT_TC1);

	 //   UARTSend1(" The ADC conversion value 2 is : \r\n");
	  //  print_dex(AD_value2);



	 }


}


/**
  * @brief  Configures the different system clocks.
  * @param  None
  * @retval None
  */
void RCC_Configuration(void)
{
#if defined (STM32F10X_LD_VL) || defined (STM32F10X_MD_VL) || defined (STM32F10X_HD_VL)
  /* ADCCLK = PCLK2/2 */
  RCC_ADCCLKConfig(RCC_PCLK2_Div2); 
#else
  /* ADCCLK = PCLK2/4 */
  RCC_ADCCLKConfig(RCC_PCLK2_Div4); 
#endif
  /* Enable peripheral clocks ------------------------------------------------*/
  /* Enable DMA1 clock */
  RCC_AHBPeriphClockCmd(RCC_AHBPeriph_DMA1, ENABLE);

  /* Enable ADC1 and GPIOC clock */
  RCC_APB2PeriphClockCmd(RCC_APB2Periph_ADC1 | RCC_APB2Periph_GPIOC, ENABLE);

}

/**
  * @brief  Configures the different GPIO ports.
  * @param  None
  * @retval None
  */
void GPIO_Configuration(void)
{
  GPIO_InitTypeDef GPIO_InitStructure;

  /* Configure PC.04 (ADC Channel14) as analog input -------------------------*/
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_4;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AIN;
  GPIO_Init(GPIOC, &GPIO_InitStructure);
}

/*******************************************************************************
* Function Name  : USART1_Configuration
* Description    : Configure USART1.
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void USART1_Configuration()
{
    const char menu[] = " \r\n Welcome to CooCox!\r\n";

    /* Configure the USART1 */
    USART_InitTypeDef USART_InitStructure;

    /* Configure the GPIOs */
    GPIO_InitTypeDef GPIO_InitStructure;

    /* Enable USART1 and GPIOA clock */
    RCC_APB2PeriphClockCmd(RCC_APB2Periph_USART1 | RCC_APB2Periph_GPIOA, ENABLE);



    /* Configure USART1 Tx (PA.09) as alternate function push-pull */
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_9;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_Init(GPIOA, &GPIO_InitStructure);
        
    /* Configure USART1 Rx (PA.10) as input floating */
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_10;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
    GPIO_Init(GPIOA, &GPIO_InitStructure);

    /* USART1 configuration ------------------------------------------------------*/
    /* USART1 configured as follow:
                - BaudRate = 115200 baud    
                - Word Length = 8 Bits
                - One Stop Bit
                - No parity
                - Hardware flow control disabled (RTS and CTS signals)
                - Receive and transmit enabled
                - USART Clock disabled
                - USART CPOL: Clock is active low
                - USART CPHA: Data is captured on the middle 
                - USART LastBit: The clock pulse of the last data bit is not output to 
                                                 the SCLK pin
    */
    USART_InitStructure.USART_BaudRate = 9600;//115200;
    USART_InitStructure.USART_WordLength = USART_WordLength_8b;
    USART_InitStructure.USART_StopBits = USART_StopBits_1;
    USART_InitStructure.USART_Parity = USART_Parity_No;
    USART_InitStructure.USART_HardwareFlowControl = USART_HardwareFlowControl_None;
    USART_InitStructure.USART_Mode = USART_Mode_Rx | USART_Mode_Tx;

    USART_Init(USART1, &USART_InitStructure);
        
    /* Enable USART1 */
    USART_Cmd(USART1, ENABLE);

    /* print welcome information */
    UARTSend1(menu);
}

/*******************************************************************************
* Function Name  : UARTSend1
* Description    : Send a string to the UART.
* Input          : - pucBuffer: buffers to be printed.
* Output         : None
* Return         : none
*******************************************************************************/
void UARTSend1(const char *pucBuffer)
{
    //
    // Loop while there are more characters to send.
    //
    while(*pucBuffer != '\0')
    {
        USART_SendData(USART1, (uint16_t) *pucBuffer++);
        /* Loop until the end of transmission */
        while(USART_GetFlagStatus(USART1, USART_FLAG_TC) == RESET)
        {
        }
    }
}
void print_dex(const int ivalue)
{
    int i,j;
    char c[9];
    for(i=0; i<8; i++)
    {
        j = ivalue>>(i*4) & 0x0F;
        if(j<10)
            c[7-i] = j + '0';
        else
            c[7-i] = j + 55;
    }
    c[8] = 0;
    UARTSend1("0x");
    UARTSend1(c);
}
#ifdef  USE_FULL_ASSERT

/**
  * @brief  Reports the name of the source file and the source line number
  *         where the assert_param error has occurred.
  * @param  file: pointer to the source file name
  * @param  line: assert_param error line source number
  * @retval None
  */
void assert_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

  /* Infinite loop */
  while (1)
  {
  }
}

#endif
